#ifndef STRICT
#define STRICT
#endif

#include <math.h>
#include "MyApplication.h"
#include "GapiDrawExtension.h"
#include "resource.h"

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Exit the loop by calling Shutdown().
//-----------------------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPTSTR pCmdLine, int nCmdShow)
{
	GDAPPCONFIG config;
	::ZeroMemory(&config, sizeof(GDAPPCONFIG));

	config.hInstance           = hInst;
	config.pAppTitle           = _T("Simple");
	config.dwWindowIcon        = IDI_APP;
	config.dwDisplayFlags      = 0;
	config.dwDisplayMode       = GDDISPMODE_NORMAL; // No display rotation
	config.dwMaxFPS            = 30; // Limit number of calls to ProcessNextFrame each second to 30

	// The following flags are only relevant if compiled with Embedded Visual C++ 4.0 or later
#if defined(_WIN32_WCE) && (_WIN32_WCE >= 420)
	config.dwDisplayFlags     |= GDDISPLAY_HI_RES_AWARE; // Enable VGA resolution
#endif

	// The following flags are only relevant on stationary PCs
#ifdef _DEBUG
	config.dwDisplayFlags     |= GDDISPLAY_WINDOW;
	config.dwWindowBorder      = WS_DLGFRAME; // Set bordertype to WS_DLGFRAME to prevent window resizing
	config.dwDisplayWidth      = 240;
	config.dwDisplayHeight     = 320;
	config.dwDisplayZoomWidth  = 480;
	config.dwDisplayZoomHeight = 640;
#else
	config.dwDisplayFlags     |= GDDISPLAY_FULLSCREEN;
	config.dwDisplayWidth      = 640;
	config.dwDisplayHeight     = 480;
	config.dwDisplayHz         = 75;
#endif

	// Create the CGapiApplication subclass
	CMyApplication* pMyApp = new CMyApplication(config);

	// Start the main loop
	HRESULT hr = -1;
	if (pMyApp)
	{
		hr = pMyApp->Run();
		delete pMyApp;
	}

	// Exit application
	return hr;
}

CMyApplication::CMyApplication(const GDAPPCONFIG& config) : CGapiApplication(config)
{
	m_font = new CGapiBitmapFont(m_pGapiDraw);
	m_ipaq = new CGapiSurface(m_pGapiDraw);
	m_taskbar = new CGapiSurface(m_pGapiDraw);
	m_cursor = new CGapiCursor(m_pGapiDraw);
	mode = 1;
}

CMyApplication::~CMyApplication()
{
	delete m_font;
	delete m_ipaq;
	delete m_taskbar;
	delete m_cursor;
}

HRESULT CMyApplication::InitInstance()
{
	return S_OK;
}

HRESULT CMyApplication::ExitInstance()
{
	return S_OK;
}

HRESULT CMyApplication::OnMinimize()
{
	return S_OK;
}

HRESULT CMyApplication::OnRestore()
{
	return S_OK;
}

HRESULT CMyApplication::CreateSysMemSurfaces(CGapiDisplay* pDisplay, HINSTANCE hInstance)
{
	const COLORREF TRANSPARENTCOLOR = RGB(255, 0, 255);

	// Create the PDA surface in system memory
	// (reading from sysmem is much faster than reading from vidmem)
	m_ipaq->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDR_IPAQ, _T("PNG"));
	m_ipaq->SetColorKey(TRANSPARENTCOLOR);

	// Create the cursor in system memory (RGBA surfaces can only be stored in sysmem)
	m_cursor->CreateSurface(GDRGBASURFACE_SYSTEMMEMORY, hInstance, IDR_CURSOR, _T("PNG"));
	m_cursor->CreateCursor(0, 20, 2);
	m_cursor->SetHotSpot(3, 4);

	// Create the bitmapped font in system memory
	m_font->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDR_SIMPLEFONT, _T("PNG"));
	m_font->CreateFont(NULL, TRANSPARENTCOLOR, GDCREATEFONT_SIMPLEBITMAP, NULL);

	// Create the taskbar in system memory
	m_taskbar->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDR_TASKBAR, _T("PNG"));
	m_taskbar->SetColorKey(TRANSPARENTCOLOR);

	// Create the taskbar buttons
	const DWORD nBtnWidth = 24;
	DWORD dwXMinimize = pDisplay->GetWidth()-m_taskbar->GetWidth();
	DWORD dwXRotate   = dwXMinimize + nBtnWidth;
	DWORD dwXClose    = dwXRotate   + nBtnWidth;
	::SetRect(&m_btnMinimize, dwXMinimize, 0, dwXMinimize + nBtnWidth, m_taskbar->GetHeight());
	::SetRect(&m_btnRotate,   dwXRotate,   0, dwXRotate + nBtnWidth,   m_taskbar->GetHeight());
	::SetRect(&m_btnClose,    dwXClose,    0, dwXClose + nBtnWidth,    m_taskbar->GetHeight());

	return S_OK;
}

HRESULT CMyApplication::CreateVidMemSurfaces(CGapiDisplay* pDisplay, HINSTANCE hInstance)
{
	return S_OK;
}

HRESULT CMyApplication::Panel2(CGapiSurface* pBackBuffer)
{
	pBackBuffer->FillRect(NULL, RGB(128, 128, 128), 0, NULL);
	return GD_OK;
}

HRESULT CMyApplication::Panel1(CGapiSurface* pBackBuffer)
{

	// Set clipping to entire surface (for the cursor)
	pBackBuffer->SetClipper(NULL);

	static int nRotAngle = 0;
	static double nRotScale = -360;

	DWORD dwScreenWidth = pBackBuffer->GetWidth();
	DWORD dwScreenHeight = pBackBuffer->GetHeight();

	// Clean up background after last pass
	pBackBuffer->FillRect(NULL, RGB(128, 128, 128), 0, NULL);

	int nScale = 80 + (int) (cos(sin(nRotScale*3.14159/180.0)) * 80.0);
	nRotScale += 5;
	nRotAngle += 100;
	if (nRotScale >= 360) nRotScale = -360;
	if (nRotAngle >= 36000) nRotAngle = 0;

	// Draw a keysourced, rotated and scaled PDA
	GDBLTFX bltfx;
	bltfx.nRotationAngle = nRotAngle;
	bltfx.dwRotationScale = nScale;
	pBackBuffer->Blt(NULL, m_ipaq, NULL,
		GDBLT_KEYSRC |
		GDBLT_ROTATIONANGLE |
		GDBLT_ROTATIONSCALE,
		&bltfx);

	_TCHAR str[128];

	FLOAT nActualFrameTime = 0;
	m_pTimer->GetActualFrameTime(&nActualFrameTime);
	_stprintf(str, _T("Current frame: [%.3f ms]"), nActualFrameTime);
	DrawFrameInfo(pBackBuffer, dwScreenHeight - 40, str, RGB(170, 90, 60));

	FLOAT nFrameRate = 0;
	m_pTimer->GetActualFrameRate(&nFrameRate);
	_stprintf(str, _T("Current framerate: [%.3f fps]"), nFrameRate);
	DrawFrameInfo(pBackBuffer, dwScreenHeight - 20, str, RGB(90, 60, 170));

	// Draw a shaded rectangle
	RECT rect;
	GDFILLRECTFX rectfx;
	::SetRect(&rect, 0, (dwScreenHeight>>1) - m_font->GetHeight()-10, dwScreenWidth, (dwScreenHeight>>1) + m_font->GetHeight()+10);
	rectfx.dwOpacity = 128; // Use 50% quick opacity
	pBackBuffer->FillRect(&rect, RGB(0, 0, 0), GDFILLRECT_OPACITY, &rectfx);

	// Draw some bitmapped text
	DWORD dwDisplayMode;
	m_pDisplay->GetDisplayMode(&dwDisplayMode);
	switch (dwDisplayMode)
	{
	case GDDISPMODE_NORMAL:
		_tcscpy(str, _T("Display Mode: NORMAL"));
		break;
	case GDDISPMODE_ROTATE90CCW:
		_tcscpy(str, _T("Display Mode: ROTATE90CCW"));
		break;
	case GDDISPMODE_ROTATE180:
		_tcscpy(str, _T("Display Mode: ROTATE180"));
		break;
	case GDDISPMODE_ROTATE90CW:
		_tcscpy(str, _T("Display Mode: ROTATE90CW"));
		break;
	}

	pBackBuffer->DrawText(dwScreenWidth>>1, (dwScreenHeight>>1) - m_font->GetHeight(), _T("Welcome to GapiDraw!"), m_font, GDDRAWTEXT_CENTER, 0, NULL, NULL);
	pBackBuffer->DrawText(dwScreenWidth>>1, dwScreenHeight>>1, str, m_font, GDDRAWTEXT_CENTER, 0, NULL, NULL);

	// Add the taskbar
	pBackBuffer->BltFast(pBackBuffer->GetWidth()-m_taskbar->GetWidth(), 0, m_taskbar, NULL, GDBLTFAST_KEYSRC, NULL);

#ifndef _WIN32_WCE
	// Draw a cursor if full screen on Stationary PCs
	if (IsFullscreen())
	{
		m_cursor->DrawCursor(pBackBuffer);
	}
#endif

	return S_OK;
}


HRESULT CMyApplication::ProcessNextFrame(CGapiSurface* pBackBuffer, DWORD dwFlags)
{
	switch(mode)
	{
		case 1:
			Panel1(pBackBuffer);
			break;
		case 2:
			Panel2(pBackBuffer);
			break;
		default:
			break;
	}

	return S_OK;
}

HRESULT CMyApplication::DrawFrameInfo(CGapiSurface* pSurface, DWORD dwY, const _TCHAR* pText, COLORREF color)
{
	DWORD strX = 30;
	DWORD strWidth;
	RECT rect;

	GDFILLRECTFX fillrectfx;
	fillrectfx.dwOpacity = 128;

	m_pDisplay->GetSystemFontBorder()->GetStringWidth(pText, &strWidth);
	::SetRect(&rect, 0, dwY-1, strX+strWidth+3, dwY+8);

	pSurface->DrawLine(0, dwY-2, strX+strWidth+3, dwY-2, RGB(0, 0, 0), 0, NULL);
	pSurface->DrawLine(strX+strWidth+3, dwY-2, strX+strWidth+3, dwY+8, RGB(0, 0, 0), 0, NULL);
	pSurface->DrawLine(0, dwY+8, strX+strWidth+3, dwY+8, RGB(0, 0, 0), 0, NULL);
	pSurface->FillRect(&rect, color, GDFILLRECT_OPACITY, &fillrectfx);
	pSurface->DrawText(strX, dwY, pText, m_pDisplay->GetSystemFontBorder(), 0, NULL, 0, NULL);

	return S_OK;
}

HRESULT CMyApplication::KeyDown(DWORD dwKey, GDKEYLIST& keylist)
{
	// Minimize window
	if (dwKey == keylist.vkA)
	{
		Minimize();
	}
	// Rotate window
	else if (dwKey == keylist.vkB)
	{
		DWORD dwMode;
		m_pDisplay->GetDisplayMode(&dwMode);
		switch (dwMode)
		{
		case GDDISPMODE_NORMAL:
			dwMode = GDDISPMODE_ROTATE90CCW;
			break;
		case GDDISPMODE_ROTATE90CCW:
			dwMode = GDDISPMODE_ROTATE180;
			break;
		case GDDISPMODE_ROTATE180:
			dwMode = GDDISPMODE_ROTATE90CW;
			break;
		case GDDISPMODE_ROTATE90CW:
			dwMode = GDDISPMODE_NORMAL;
			break;
		}
		SetDisplayMode(dwMode);
	}
	// Exit application
	else if (dwKey == keylist.vkC)
	{
		Shutdown();
	}
	return S_OK;
}

HRESULT CMyApplication::KeyUp(DWORD dwKey, GDKEYLIST& keylist)
{
	return S_OK;
}

HRESULT CMyApplication::StylusDown(POINT p)
{
	// Minimize window
	if (::PtInRect(&m_btnMinimize, p))
	{
		Minimize();
	}
	// Rotate window
	else if (::PtInRect(&m_btnRotate, p))
	{
		DWORD dwMode;
		m_pDisplay->GetDisplayMode(&dwMode);
		switch (dwMode)
		{
		case GDDISPMODE_NORMAL:
			dwMode = GDDISPMODE_ROTATE90CCW;
			break;
		case GDDISPMODE_ROTATE90CCW:
			dwMode = GDDISPMODE_ROTATE180;
			break;
		case GDDISPMODE_ROTATE180:
			dwMode = GDDISPMODE_ROTATE90CW;
			break;
		case GDDISPMODE_ROTATE90CW:
			dwMode = GDDISPMODE_NORMAL;
			break;
		}
		
		SetDisplayMode(dwMode);

		if(mode == 1)
			mode = 2;
		else
			mode = 1;

	}
	// Exit application
	else if (::PtInRect(&m_btnClose, p))
	{
		Shutdown();
	}

	return S_OK;
}

HRESULT CMyApplication::StylusUp(POINT p)
{
	return S_OK;
}

HRESULT CMyApplication::StylusDblClk(POINT p)
{
	return S_OK;
}

HRESULT CMyApplication::StylusMove(POINT p)
{
	m_cursor->SetPosition(p.x, p.y);
	return S_OK;
}

